package com.sun.jmx.mbeanserver;

import java.lang.ref.Reference;
import java.lang.ref.ReferenceQueue;
import java.lang.ref.WeakReference;
import java.util.Map;

class WeakIdentityHashMap<K, V>
{
    private Map<WeakReference<K>, V> map;
    private ReferenceQueue<K> refQueue;
    
    private WeakIdentityHashMap() {
        this.map = Util.newMap();
        this.refQueue = new ReferenceQueue();
    }
    
    static <K, V> WeakIdentityHashMap<K, V> make() {
        return new WeakIdentityHashMap();
    }
    
    V get(K k) {
        this.expunge();
        return this.map.get(this.makeReference(k));
    }
    
    public V put(K k, V v) {
        this.expunge();
        if (k == null) {
            throw new IllegalArgumentException("Null key");
        }
        return this.map.put(this.makeReference(k, this.refQueue), v);
    }
    
    public V remove(K k) {
        this.expunge();
        return this.map.remove(this.makeReference(k));
    }
    
    private void expunge() {
        Reference<Object> poll;
        while ((poll = this.refQueue.poll()) != null) {
            this.map.remove(poll);
        }
    }
    
    private WeakReference<K> makeReference(K k) {
        return new WeakIdentityHashMap.IdentityWeakReference(k);
    }
    
    private WeakReference<K> makeReference(K k, ReferenceQueue<K> referenceQueue) {
        return new WeakIdentityHashMap.IdentityWeakReference(k, referenceQueue);
    }
    
    private static class IdentityWeakReference<T> extends WeakReference<T>
    {
        private final int hashCode;
        
        IdentityWeakReference(T t) {
            this(t, null);
        }
        
        IdentityWeakReference(T referent, ReferenceQueue<T> q) {
            super(referent, q);
            this.hashCode = ((referent == null) ? 0 : System.identityHashCode(referent));
        }
        
        @Override
        public boolean equals(Object o) {
            if (this == o) {
                return true;
            }
            if (!(o instanceof WeakIdentityHashMap.IdentityWeakReference)) {
                return false;
            }
            WeakIdentityHashMap.IdentityWeakReference<Object> identityWeakReference = (WeakIdentityHashMap.IdentityWeakReference)o;
            Object value = this.get();
            return value != null && value == identityWeakReference.get();
        }
        
        @Override
        public int hashCode() {
            return this.hashCode;
        }
    }
}
